"""
 Copyright (C) 2014 Maruf Maniruzzaman
 Website: http://cosmosframework.com
 Author: Maruf Maniruzzaman
 License :: OSI Approved :: MIT License
"""


"""
 The private key should not leave the location where it was generated. But in datacenter environment it may be desirable
 to store the certificate in a central certificate store and move around different machines in the datacenter. Like a
 webserver machine may be redeployed completely and we may need to copy the private key to that new machine. In order
 to securely do this securely, the deployment service may generate a set of keys dep_priv_key, dep_pub_key and send the
 public key to the certificate manager. The certificate manager should verify requesting machines integrity and encrypt
 the private ker with supplied public key of the client. This can be done using already established protocol like
 HTTPS using a client certificate to verify the client.

"""


"""
 Copyright (C) 2016 Maruf Maniruzzaman
 Website: http://cosmosframework.com
 Author: Maruf Maniruzzaman
 License :: OSI Approved :: MIT License
"""

import OpenSSL.crypto as crypto


# sudo apt-get install build-essential libssl-dev libffi-dev python-dev
# pip install pyOpenSSL


def generate_certificate(key_length, cn, organizational_unit, organization, locality, state_or_province, country):
    request = crypto.X509Req()

    subject = request.get_subject()

    subject.CN = cn
    subject.organizationalUnitName = organizational_unit
    subject.organizationName = organization
    subject.localityName = locality
    subject.stateOrProvinceName = state_or_province
    subject.countryName = country

    key = generate_key(key_length)

    request.set_pubkey(key)
    request.sign(key, "sha256")

    csr = crypto.dump_certificate_request(crypto.FILETYPE_PEM, request)
    private_key = crypto.dump_privatekey(crypto.FILETYPE_PEM, key)

    certificate = {"private_key": private_key, "csr": csr, "key_length": key_length}

    return certificate


def generate_key(key_length):
    key = crypto.PKey()
    key.generate_key(crypto.TYPE_RSA, key_length)
    return key


def sign_certificate(req, issuerCert, issuerKey, serial, notBefore, notAfter, digest="md5"):
    """
    Generate a certificate given a certificate request.
    Arguments: req        - Certificate reqeust to use
               issuerCert - The certificate of the issuer
               issuerKey  - The private key of the issuer
               serial     - Serial number for the certificate
               notBefore  - Timestamp (relative to now) when the certificate
                            starts being valid
               notAfter   - Timestamp (relative to now) when the certificate
                            stops being valid
               digest     - Digest method to use for signing, default is md5
    Returns:   The signed certificate in an X509 object
    """

    # from https://github.com/msabramo/pyOpenSSL/blob/master/examples/certgen.py
    # -*- coding: latin-1 -*-
    #
    # Copyright (C) AB Strakt
    # Copyright (C) Jean-Paul Calderone
    # License : Apache 2.0

    cert = crypto.X509()
    cert.set_serial_number(serial)
    cert.gmtime_adj_notBefore(notBefore)
    cert.gmtime_adj_notAfter(notAfter)
    cert.set_issuer(issuerCert.get_subject())
    cert.set_subject(req.get_subject())
    cert.set_pubkey(req.get_pubkey())
    cert.sign(issuerKey, digest)
    return cert


"""
 Copyright (C) 2016 Maruf Maniruzzaman
 Website: http://cosmosframework.com
 Author: Maruf Maniruzzaman
 License :: OSI Approved :: MIT License
"""

from motor.frameworks import tornado
from tornado import gen

from cosmos.certmgr.certfactory import generate_certificate
from cosmos.common.constants import COSMOS_CERT_STORE_OBJECT_NAME
from cosmos.rbac.object import SYSTEM_USER
from cosmos.service.requesthandler import RequestHandler

DEFAULT_KEY_LEN = 2048


class CertificateVaultHandler(RequestHandler):
    @gen.coroutine
    def post(self, operation):

        if operation == "generate":
            country = self.get_argument("C")
            state_or_province = self.get_argument("ST")
            locality = self.get_argument("L")
            organization = self.get_argument("O")
            organizational_unit = self.get_argument("OU")
            cn = self.get_argument("CN")
            key_len = int(self.get_argument("key_length", default=DEFAULT_KEY_LEN))

            cert = generate_certificate(key_len, cn, organizational_unit, organization, locality, state_or_province, country)

            id = yield self.insert_cert_in_vault(cert)
            self.write(id)
            return

        raise tornado.web.HTTPError(404, "Not found")

    @gen.coroutine
    def insert_cert_in_vault(self, cert):
        obj_serv = self.settings['object_service']
        promise = obj_serv.insert(SYSTEM_USER, COSMOS_CERT_STORE_OBJECT_NAME, cert)
        req_id = yield promise
        raise gen.Return(req_id)



